#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>

#include "ffft.h"
#include "../ixtdmv11hw.h"
#include "../ixtdm_com.h"

#define KEY_PRESSED (!(PINB&(1<<PB0)))
#define LED3_TOGGLE PORTD^=(1<<PD7)


// FFFT
int16_t capture1[FFT_N], capture2[FFT_N];	/* Wave captureing buffer */
complex_t bfly_buff[FFT_N];					/* FFT buffer */
uint16_t spektrum[FFT_N/2];					/* Spectrum output buffer */
int16_t * readbuffer = capture1;
int16_t * writebuffer = capture2;
int16_t * tempbuffer;
uint16_t writeindex = 0;
// PWM
uint8_t r = 0, g = 0, b = 0;

/*! \brief Flags for status information
 */
volatile struct GLOBAL_FLAGS {
  uint8_t writeBufferFull:1;
  uint8_t readBufferFull:1;
  uint8_t dummy:6;
} gFlags = {0, 0, 0};

// FFT Ergebnis auf PWM skalieren
uint8_t sc(uint16_t i)
{
	uint8_t x;
	if(i>=500) i -=500;
	i >>= 4;
	if( i > 255 )
	{
		x = 255;
	}
	else
	{
		x = i;
	}

	return x;
}

// Hauptfunktion
int main(void)
{
	// HW
	init_hardware();

	// Kommunikation
	init_com(0);

	// Timer 0: fr ADC Trigger -> ca. alle 1ms Overflow (16e6/256/64)
	TCCR0A = 0;
	TCCR0B = (0<<CS02) + (1<<CS01) + (1<<CS00);

	// ADC
	// Trigger Src: T0 OVF, Prescaler: 128
	// MICIN an ADC7, Linksausgerichtetes Ergebnis!
	ADMUX  = (0<<REFS1) + (1<<REFS0) + (1<<ADLAR) + 7;
	ADCSRA = (1<<ADEN) + (0<<ADSC) + (1<<ADATE) + (1<<ADIF) + (1<<ADIE) + 7;
	ADCSRB = (1<<ADTS2) + (0<<ADTS1) + (0<<ADTS0);

	// Interrupts an
	sei();

	// Main Loop
	while(1)
	{
		// Gibt es Arbeit?
		if(gFlags.readBufferFull)
		{
			// FFT
			fft_input(readbuffer, bfly_buff);
			fft_execute(bfly_buff);
			fft_output(bfly_buff, spektrum);
			// Fertig
			gFlags.readBufferFull = 0;
			// Effekt + Moving Average
			r = (r + sc(spektrum[5]))/2; // f = x*15 Hz
			g = (g + sc(spektrum[10]))/2;
			b = (b + sc(spektrum[15]))/2;
			// Daten an alle PWM-Clients schicken
			com_send(0,1,r);
			com_send(0,2,g);
			com_send(0,3,b);
		}
	}
}

// ADC Interrupt
ISR(ADC_vect)
{
	// ADC-Wert sichern, falls irgendwo Platz
	if(!gFlags.writeBufferFull)
	{
		writebuffer[writeindex] = ADCW - 32768; // links ausgerichteten AD-Wert auf Mitte skalieren
		writeindex++;
		// Buffer voll?
		if(writeindex == FFT_N)
		{
			writeindex = 0;
			// Buffer tauschen, falls mglich
			if(!gFlags.readBufferFull)
			{
				tempbuffer = writebuffer;
				writebuffer = readbuffer;
				readbuffer = tempbuffer;

				gFlags.readBufferFull = 1;
			}
			else
			{
				// Ansonsten warten
				gFlags.writeBufferFull = 1;
				//LED1_ON;
			}
		}
	}
	else
	{
		// Weitermachen mglich?
		if(!gFlags.readBufferFull)
		{
			tempbuffer = writebuffer;
			writebuffer = readbuffer;
			readbuffer = tempbuffer;

			gFlags.readBufferFull = 1;
			gFlags.writeBufferFull = 0;

			//LED1_OFF;
		}
	}
	// T0 OVF Flag zurcksetzen
	TIFR0 = (1<<TOV0);
}
