#include <avr/io.h>

#include "helpers.h"

// HSV zu RGB umrechnen
// H: Farbton, 0-1535, rot-orange-gelb-grn-hellblau-dunkelblau-violett-rot
// S: Sttigung, 0-255, wei-farbton
// V: Dunkelstufe, 0-255, schwarz-farbton
// siehe http://de.wikipedia.org/wiki/Bild:HSV-RGB-comparison.svg
color_rgb driver_hsv2rgb(color_hsv hsv)
{
	uint16_t hdiv,r,g,b,p,q,t;
	uint8_t f;
	color_rgb result;
	//HSV --> RGB laut Wikipedia
	//auf 16 Bit Integer umgebaut
	if(hsv.s == 0)
	{
		r = g = b = hsv.v*256;
	}
	else
	{
		hdiv = hsv.h / 256;
		f = (hsv.h % 256);

		p = hsv.v * (255 - hsv.s);

		q = hsv.v * ((0xFFFF - hsv.s * f)/256);
		t = hsv.v * ((0xFFFF - hsv.s * (255 - f))/256);

		switch( hdiv )
		{
			case 0:
				r = hsv.v*256; g = t; b = p; break;
			case 1:
				r = q; g = hsv.v*256; b = p; break;
			case 2:
				r = p; g = hsv.v*256; b = t; break;
			case 3:
				r = p; g = q; b = hsv.v*256; break;				
			case 4:
				r = t; g = p; b = hsv.v*256; break;					
			default:								
				r = hsv.v*256; g = p; b = q; break;								
		}
	}
	//skalieren
	result.r = (uint8_t)(r/256);
	result.g = (uint8_t)(g/256);
	result.b = (uint8_t)(b/256);

	//zurck
	return result;
}
