#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "ixtdmv11hw.h"
#include "ixtdm_com.h"


// Empfangsringpuffer
t_com_recvdata recvdata[COM_RECVBUFFER];
volatile uint8_t recv_read = 0, recv_write = 0;
uint8_t address = 0;
// Empfangszustand
volatile uint8_t recv_state = 0;

// Initialisieren mit Clientadresse
void init_com(uint8_t addr)
{
	// Clientadresse
	address = addr;

	// Hardware
	RS485_RECEIVE;
	UBRR0 = UBRR_VAL;
	UCSR0A = 0;
	UCSR0B = (1<<RXCIE0) | (0<<TXCIE0) | (0<<UDRIE0) | (1<<RXEN0) | (1<<TXEN0) | (0<<UCSZ02);
	UCSR0C = (0<<UMSEL01) | (0<<UMSEL00) | (0<<UPM01) | (0<<UPM00) | (0<<USBS0) | (1<<UCSZ01) | (1<<UCSZ00);

	// Interrupts an
	sei();
}

// Senden
void com_send(uint8_t addr, uint8_t channel, uint8_t data)
{
	// UART bereit?
	while((UCSR0A & (1<<UDRE0)) == 0);

	// Senden!
	RS485_TRANSMIT;
	_delay_us(50);

	// Start-Puls (DMX-like)
	UCSR0B &= ~(1<<TXEN0);
	_delay_us(COM_PULSE);
	UCSR0B |= (1<<TXEN0);
	_delay_us(50);
	
	// Adresse, Kanal, Daten
	UDR0 = addr;
	while((UCSR0A & (1<<UDRE0)) == 0);
	UDR0 = channel;
	while((UCSR0A & (1<<UDRE0)) == 0);
	UCSR0A |= (1<<TXC0);
	UDR0 = data;
	while((UCSR0A & (1<<TXC0)) == 0);
	_delay_us(5);

	// Fertig
	RS485_RECEIVE;
}

// Empfangen via Interrupt
ISR(USART_RX_vect)
{
	uint8_t dummy;

	// War es ein Startpuls?
	if((UCSR0A & (1<<FE0)))
	{
		dummy = UDR0;
		recv_state = 1;
	}
	else
	{
		dummy = UDR0;
		// vorher Startpuls bekommen und Puffer nicht voll?
		if(recv_state != 0 &&
		   ((recv_write + 1) % COM_RECVBUFFER) != recv_read)
		{
			switch(recv_state)
			{
				case 1:
					// Adresse
					if(dummy == 0 || dummy == address)
					{
						recv_state = 2;
						recvdata[recv_write].flag_broadcast = dummy == 0 ? 1 : 0;
						//LED3_ON;
					} else {
						recv_state = 0;
					}
					break;
				case 2:
					// Kanal
					recvdata[recv_write].channel = dummy;
					recv_state = 3;
					break;
				case 3:
					// Daten
					recvdata[recv_write].data = dummy;
					// fertig!
					recv_write = (recv_write + 1) % COM_RECVBUFFER;
					recv_state = 0;
					//LED3_OFF;
					break;
				default:
					recv_state = 0;
					break;
			}
		}
		else
		{
			recv_state = 0;
		}
	}
}

// Nachschauen, ob etwas empfangen wurde
uint8_t com_get(t_com_recvdata * data)
{
	if(recv_read == recv_write)
	{
		// Puffer leer
		return 0;
	}
	else
	{
		*data = recvdata[recv_read];
		recv_read = (recv_read + 1) % COM_RECVBUFFER;
		return 1;
	}

}
